/*
**    TP CPE Lyon
**    Copyright (C) 2014 Damien Rohmer
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**   This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#include "matrix1x4.hpp"

#include "matrix4x1.hpp"
#include "matrix4.hpp"

#include <cmath>
#include <iostream>
#include <sstream>


namespace cpe
{

matrix1x4::matrix1x4()
{
    m[0]=0;m[1]=0;m[2]=0;m[3]=0;
}

matrix1x4::matrix1x4(float x00,float x01,float x02,float x03)
{
    m[0]=x00;m[1]=x01;m[2]=x02;m[3]=x03;
}

float matrix1x4::operator()(const size_t& k1,const size_t& k2) const
{
    assert_size(k1,k2);
    return m[k2];
}
float& matrix1x4::operator()(const size_t& k1,const size_t& k2)
{
    assert_size(k1,k2);
    return m[k2];
}

matrix1x4 matrix1x4::operator+(const matrix1x4& m2) const
{
    const float* pm2=m2.m;

    return matrix1x4(m[0]+pm2[0],m[1]+pm2[1],m[2]+pm2[2],m[3]+pm2[3]);
}

matrix1x4 operator+(float s,const matrix1x4& m)
{
    matrix1x4 temp=m;
    temp+=s;
    return temp;
}

matrix1x4 matrix1x4::operator+(float s) const
{
    matrix1x4 temp=*this;
    temp+=s;
    return temp;
}

matrix1x4 matrix1x4::operator-(const matrix1x4& m2) const
{
    matrix1x4 temp=*this;
    temp-=m2;
    return temp;
}

matrix1x4 operator-(float s,const matrix1x4& m)
{
    const float* pm=m.pointer();
    return matrix1x4(s-pm[0],s-pm[1],s-pm[2],s-pm[3]);
}

matrix1x4 matrix1x4::operator-(float s) const
{
    return matrix1x4(m[0]-s,m[1]-s,m[2]-s,m[3]-s);
}

matrix1x4 operator*(float s,const matrix1x4& m)
{
    matrix1x4 temp=m;
    temp*=s;
    return temp;
}

matrix1x4 matrix1x4::operator*(float s) const
{
    matrix1x4 temp=*this;
    temp*=s;
    return temp;
}


float matrix1x4::operator*(const matrix4x1& m2) const
{
    const float* pm2=m2.pointer();

    return m[0]*pm2[0]+m[1]*pm2[1]+m[2]*pm2[2]+m[3]*pm2[3];
}

matrix1x4 matrix1x4::operator/(float s) const
{
    return matrix1x4(m[0]/s,m[1]/s,m[2]/s,m[3]/s);
}

matrix1x4& matrix1x4::operator+=(const matrix1x4& _m)
{
    const float* pm=_m.m;
    for(unsigned int k=0;k<4;++k)
        m[k] += pm[k];
    return *this;
}

matrix1x4& matrix1x4::operator+=(float s)
{
    for(unsigned int k=0;k<4;++k)
        m[k] += s;
    return *this;
}

matrix1x4& matrix1x4::operator-=(const matrix1x4& _m)
{
    const float* pm=_m.m;
    for(unsigned int k=0;k<4;++k)
        m[k] -= pm[k];
    return *this;
}

matrix1x4& matrix1x4::operator-=(float s)
{
    for(unsigned int k=0;k<4;++k)
        m[k] -= s;
    return *this;
}

matrix1x4& matrix1x4::operator*=(float s)
{
    for(unsigned int k=0;k<4;++k)
        m[k] *= s;
    return *this;
}

matrix1x4& matrix1x4::operator/=(float s)
{
    for(unsigned int k=0;k<4;++k)
        m[k] /= s;
    return *this;
}

matrix1x4 matrix1x4::operator-() const
{
    return matrix1x4(-m[0],-m[1],-m[2],-m[3]);
}
matrix1x4 matrix1x4::product_compontentwise(const matrix1x4& _m) const
{
    const float* pm=_m.m;
    return matrix1x4(m[0]*pm[0],m[1]*pm[1],m[2]*pm[2],m[3]*pm[3]);
}

matrix1x4& matrix1x4::product_compontentwise_internal(const matrix1x4& _m)
{
    const float* pm=_m.m;
    for(unsigned int k=0;k<4;++k)
        m[k]=pm[k];
    return *this;
}

std::ostream& operator<<(std::ostream& stream,const matrix1x4& _m)
{
    const float* pm=_m.pointer();
    stream<<pm[0]<<","<<pm[1]<<"\n"<<pm[2]<<","<<pm[3]<<"\n";
    return stream;
}

void matrix1x4::assert_size(const size_t& k1,const size_t& k2) const
{
    if(k1>1 || k2>1)
    {
        std::stringstream stream;
        stream<<"matrix2::assert_size("<<k1<<","<<k2<<") > (1,1)";
        throw exception_cpe(stream.str(),MACRO_EXCEPTION_PARAMETER);
    }
}

const float* matrix1x4::pointer() const
{
    return m;
}
float* matrix1x4::pointer_unprotected()
{
    return m;
}
std::vector<float> matrix1x4::to_vector() const
{
    std::vector<float> m3d(4);
    for(unsigned int kx=0;kx<4;++kx)
        m3d[kx]=m[kx];
    return m3d;
}

matrix4x1 matrix1x4::transposed() const
{
    return matrix4x1(m[0],m[1],m[2],m[3]);
}

matrix1x4 matrix1x4::operator*(const matrix4 m_) const
{
    const float* pm=m_.pointer();
    return matrix1x4(m[0]*pm[0]+m[1]*pm[1]+m[2]*pm[2]+m[3]*pm[3],
            m[0]*pm[4]+m[1]*pm[5]+m[2]*pm[6]+m[3]*pm[7],
            m[0]*pm[8]+m[1]*pm[9]+m[2]*pm[10]+m[3]*pm[11],
            m[0]*pm[12]+m[1]*pm[13]+m[2]*pm[14]+m[3]*pm[15]);
}

}
