/*
**    TP CPE Lyon
**    Copyright (C) 2014 Damien Rohmer
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**   This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#pragma once

#ifndef matrix4x1_HPP
#define matrix4x1_HPP

#include <iostream>
#include <vector>

#include "../libcommon/exception_cpe.hpp"

namespace cpe
{

class matrix1x4;
class matrix4;

/** \brief Matrix 1x4 */
class matrix4x1
{

public:

    // ********************************************* //
    // ********************************************* //
    //  CONSTRUCTORS
    // ********************************************* //
    // ********************************************* //

    /** \brief empty constructor (identity) */
    matrix4x1();
    /** \brief direct constructor with every value */
    matrix4x1(float x00,float x01,float x02,float x03);



    // ********************************************* //
    // ********************************************* //
    //  Element access
    // ********************************************* //
    // ********************************************* //

    /** \brief Access to the k_th entry (k in [0,1]) */
    float operator()(const size_t& k1,const size_t& k2) const;
    /** \brief Access to the k_th entry (k in [0,1]) */
    float& operator()(const size_t& k1,const size_t& k2);

    /** \brief fast pointer access
            \warning unprotected access */
    const float* pointer() const;
    /** \brief fast pointer access
            \warning unprotected access */
    float* pointer_unprotected();



    // ********************************************* //
    // ********************************************* //
    //  Math operator
    // ********************************************* //
    // ********************************************* //

    /** \brief + operator */
    matrix4x1 operator+(const matrix4x1& m2) const;
    /** \brief + operator */
    matrix4x1 operator+(float s) const;

    /** \brief - operator */
    matrix4x1 operator-(const matrix4x1& m2) const;
    /** \brief - operator */
    matrix4x1 operator-(float s) const;


    /** \brief multiply by a scalar operator */
    matrix4x1 operator*(float s) const;
    /** \brief matrix mulitplication */
    float operator*(const matrix1x4& m2) const;
    /** \brief divide by a scalar operator */
    matrix4x1 operator/(float s) const;


    /** \brief internal + */
    matrix4x1& operator+=(const matrix4x1& m);
    /** \brief internal + */
    matrix4x1& operator+=(float s);
    /** \brief internal - */
    matrix4x1& operator-=(const matrix4x1& m);
    /** \brief internal - */
    matrix4x1& operator-=(float s);
    /** \brief internal * */
    matrix4x1& operator*=(float s);
    /** \brief internal / */
    matrix4x1& operator/=(float s);

    /** \brief unary negation */
    matrix4x1 operator-() const;

    /** \brief does componentwise multiplication */
    matrix4x1 product_compontentwise(const matrix4x1& m) const;
    /** \brief does componentwise multiplication */
    matrix4x1& product_compontentwise_internal(const matrix4x1& m);

    /** \brief transpose matrix */
    matrix4x1 transposed() const;







    // ********************************************* //
    // ********************************************* //
    //  Projection matrix
    // ********************************************* //
    // ********************************************* //

    /** \brief convert 1x4 matrix into a vector of float of size 4 */
    std::vector<float> to_vector() const;

private:

    // ********************************************* //
    // ********************************************* //
    //  Internal parameters
    // ********************************************* //
    // ********************************************* //

    /** \brief internal storage of the matrix */
    float m[4];


    // ********************************************* //
    // ********************************************* //
    //  Helper
    // ********************************************* //
    // ********************************************* //

    /** \brief assert that a size_t belongs to 0x[[0,3]] */
    void assert_size(const size_t& k1,const size_t& k2) const;

};

/** \brief + operator */
matrix4x1 operator+(float s,const matrix4x1& m);
/** \brief - operator */
matrix4x1 operator-(float s,const matrix4x1& m);
/** \brief multiply by a scalar operator */
matrix4x1 operator*(float s,const matrix4x1& m);
/** \brief multiply with a matrix4 */
matrix4x1 operator*(const matrix4& m1,const matrix4x1& m2);


// ********************************************* //
// ********************************************* //
//  Output
// ********************************************* //
// ********************************************* //

/** \brief output the vector in ostream */
std::ostream& operator<<(std::ostream& stream,const matrix4x1& m);






}

#endif
