/*
**    TP CPE Lyon
**    Copyright (C) 2014 Damien Rohmer
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**   This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#include "parametric_surface.hpp"


#include <cmath>
#include <cstdlib>
#include <fstream>
#include <sstream>



namespace cpe
{


void parametric_surface::resize(int Nu_param,int Nv_param)
{
    if(Nu_param<=0 || Nv_param<=0)
        throw cpe::exception_cpe("Invalid value for Nu and Nv",MACRO_EXCEPTION_PARAMETER);

    clear();

    Nu=Nu_param;
    Nv=Nv_param;

    //build vertices
    for(int ku=0;ku<Nu;++ku)
    {
        float u=ku/static_cast<float>(Nu-1);
        for(int kv=0;kv<Nv;++kv)
        {
            float v=kv/static_cast<float>(Nv-1);
            add_vertex(vec3(u,v,0.0f));
        }
    }

    //build connectivity
    for(int ku=0;ku<Nu-1;++ku)
    {
        for(int kv=0;kv<Nv-1;++kv)
        {
            int offset=kv+Nv*ku;
            add_triangle(offset,offset+1,offset+Nv+1);
            add_triangle(offset,offset+Nv+1,offset+Nv);
        }
    }


}

const vec3& parametric_surface::operator()(int ku,int kv) const
{
    if(Nu*Nv!=static_cast<int>(number_of_vertices()))
        throw cpe::exception_cpe("Invalid value for Nu and Nv",MACRO_EXCEPTION_PARAMETER);
    if(ku<0 || ku>=Nu || kv<0 || kv>=Nv)
    {
        std::stringstream msg;
        msg<<"Value S("<<ku<<","<<kv<<") outsize valid indices with size=("<<Nu<<","<<Nv<<")";
        throw cpe::exception_cpe(msg.str(),MACRO_EXCEPTION_PARAMETER);
    }

    int offset=kv+Nv*ku;
    return vertex(offset);
}

vec3& parametric_surface::operator()(int ku,int kv)
{
    if(Nu*Nv!=static_cast<int>(number_of_vertices()))
        throw cpe::exception_cpe("Invalid value for Nu and Nv",MACRO_EXCEPTION_PARAMETER);
    if(ku<0 || ku>=Nu || kv<0 || kv>=Nv)
    {
        std::stringstream msg;
        msg<<"Value S("<<ku<<","<<kv<<") outsize valid indices with size=("<<Nu<<","<<Nv<<")";
        throw cpe::exception_cpe(msg.str(),MACRO_EXCEPTION_PARAMETER);
    }

    int offset=kv+Nv*ku;
    return vertex(offset);
}

const vec3& parametric_surface::color(int ku,int kv) const
{
    if(Nu*Nv!=static_cast<int>(number_of_vertices()))
        throw cpe::exception_cpe("Invalid value for Nu and Nv",MACRO_EXCEPTION_PARAMETER);
    if(ku<0 || ku>=Nu || kv<0 || kv>=Nv)
    {
        std::stringstream msg;
        msg<<"Value S("<<ku<<","<<kv<<") outsize valid indices with size=("<<Nu<<","<<Nv<<")";
        throw cpe::exception_cpe(msg.str(),MACRO_EXCEPTION_PARAMETER);
    }

    int offset=kv+Nv*ku;
    return mesh::color(offset);
}
vec3& parametric_surface::color(int ku,int kv)
{
    if(Nu*Nv!=static_cast<int>(number_of_vertices()))
        throw cpe::exception_cpe("Invalid value for Nu and Nv",MACRO_EXCEPTION_PARAMETER);
    if(ku<0 || ku>=Nu || kv<0 || kv>=Nv)
    {
        std::stringstream msg;
        msg<<"Value S("<<ku<<","<<kv<<") outsize valid indices with size=("<<Nu<<","<<Nv<<")";
        throw cpe::exception_cpe(msg.str(),MACRO_EXCEPTION_PARAMETER);
    }

    int offset=kv+Nv*ku;
    return mesh::color(offset);
}

int parametric_surface::size_u() const
{
    return Nu;
}
int parametric_surface::size_v() const
{
    return Nv;
}

}


